{ File   : SaveTGA.pas
  Author : Deniz Oezmen
  Created: 2007-03-24
  Changed: 2007-03-24

  Library providing functions to export data as a TGA bitmap file.
}
unit SaveTGA;

interface

type
  TColor   = packed record
               B, G, R, A: Byte;
             end;

  TPalette = packed array of TColor;

  PByte    = ^Byte;

function DumpIndexedTGA(const TGAName: string; Width, Height: Longint;
  const Palette: TPalette; Content: PByte): Boolean;

implementation

uses
  SysUtils;

type
  TTGAHeader = packed record
                 IDLength         : Byte;
                 ColorMapType     : Byte;
                 ImageType        : Byte;
                 FirstEntryIndex  : Word;
                 ColorMapLength   : Word;
                 ColorMapEntrySize: Byte;
                 XOrigin          : Word;
                 YOrigin          : Word;
                 ImageWidth       : Word;
                 ImageHeight      : Word;
                 PixelDepth       : Byte;
                 ImageDescriptor  : Byte;
               end;

function OpenTGAFile(var TGAFile: file; const TGAName: string): Boolean;
begin
  FileMode := fmOpenWrite;
  AssignFile(TGAFile, TGAName);
  {$I-}
  Rewrite(TGAFile, 1);
  {$I+}

  Result := (IOResult = 0)
end;

function DumpIndexedTGA(const TGAName: string; Width, Height: Longint;
  const Palette: TPalette; Content: PByte): Boolean;
var
  i        : Longint;
  TGAFile  : file;
  TGAHeader: TTGAHeader;
  Buffer   : array of TColor;
begin
  Result := OpenTGAFile(TGAFile, TGAName);
  if not Result then
    Exit;

  Result := Length(Palette) <= 2048;
  if not Result then
    Exit;

  // fill header structures
  with TGAHeader do
  begin
    IDLength := 0;
    ColorMapType := 0;
    ImageType := 2;
    FirstEntryIndex := 0;
    ColorMapLength := 0;
    ColorMapEntrySize := 0;
    XOrigin := 0;
    YOrigin := 0;
    ImageWidth := Width;
    ImageHeight := Height;
    PixelDepth := 32;
    ImageDescriptor := $28
  end;

  {$I-}
  BlockWrite(TGAFile, TGAHeader, SizeOf(TGAHeader));
  {$I+}
  Result := (IOResult = 0);
  if not Result then
    Exit;

  // convert data
  SetLength(Buffer, Width * Height);
  for i := 0 to Length(Buffer) - 1 do
  begin
    Buffer[i] := Palette[Byte(Content^)];
    Inc(Content)
  end;

  {$I-}
  BlockWrite(TGAFile, Buffer[0], Length(Buffer) * SizeOf(TColor));
  {$I+}
  Result := (IOResult = 0);

  Finalize(Buffer)
end;

end.
