{ File   : TEX2TGA.dpr
  Author : Deniz Oezmen
  Created: 2007-03-24
  Changed: 2007-03-25

  Star Wars: Jedi Starfighter TEX converter.
}
program TEX2TGA;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  SaveTGA;

type
  TTEXHeader = packed record
                 ID     : array[0..3] of Char;
                 Unknown: array[0..7] of Byte;
                 Width,
                 Height : Longword;  // or vice-versa?
               end;

  TBuffer    = array of Byte;

function OpenTEXFile(var TEXFile: file; TEXName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(TEXFile, TEXName);

  {$I-}
  Reset(TEXFile, 1);
  {$I+}
  OpenTEXFile := (IOResult = 0)
end;

function ReadHeader(var TEXFile: file; out TEXHeader: TTEXHeader): Boolean;
var
  BytesRead: Longword;
begin
  BlockRead(TEXFile, TEXHeader, SizeOf(TTEXHeader), BytesRead);
  ReadHeader := (SizeOf(TTEXHeader) = BytesRead) and
    (TEXHeader.ID = 'YATF')
end;

function ReadPalette(var TEXFile: file; out Palette: TPalette): Boolean;
var
  i, t: Byte;
begin
  SetLength(Palette, 256);
  {$I-}
  BlockRead(TEXFile, Palette[0], Length(Palette) * SizeOf(TColor));
  {$I+}
  Result := (IOResult = 0);

  //FIXME: Is the last byte indeed the alpha channel?

  for i := 0 to 255 do
    with Palette[i] do
    begin
      t := R;
      R := B;
      B := t
    end
end;

procedure Convert(var TEXFile: file; const TEXName: string;
  const TEXHeader: TTEXHeader);
var
  Buffer : TBuffer;
  Palette: TPalette;
begin
  if not ReadPalette(TEXFile, Palette) then
  begin
    WriteLn('  error reading palette data');
    Exit
  end;

  with TEXHeader do
    SetLength(Buffer, Width * Height);
  {$I-}
  BlockRead(TEXFile, Buffer[0], Length(Buffer));
  {$I+}
  if IOResult <> 0 then
  begin
    WriteLn('  error reading bitmap data');
    Exit
  end;

  with TEXHeader do
    if not DumpIndexedTGA(TEXName + '.tga', Width, Height, Palette, @Buffer[0])
      then
      WriteLn('  export error')
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  TEXName  : string;
  TEXFile  : file;
  TEXHeader: TTEXHeader;
  SearchRec: TSearchRec;
  PFilePara: PChar;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  // search for all files matching FilePara
  if FindFirst(PFilePara, faAnyFile - faDirectory, SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      TEXName := Dir + FileName;

      WriteLn(TEXName);

      // if this file is accessible, analyze it
      if OpenTEXFile(TEXFile, TEXName) then
      begin
        if ReadHeader(TEXFile, TEXHeader) then
          Convert(TEXFile, TEXName, TEXHeader)
        else
          WriteLn('  not a valid (or supported) TEX file');
        CloseFile(TEXFile)
      end
      else
        Writeln(TEXName, '  not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    Writeln(FilePara, '  not found')
end;

begin
  if ParamCount >= 1 then
    ParseFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.

